#! /usr/bin/perl -w

# Comditionals.pm: Handle conditionals in car file directives
# 
#    Copyright (C) 2005, Patrice Dumas
# 
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
use strict;

package CMZ::Conditionals;

use CMZ::Bool;

my $parser = new CMZ::Bool;
#$parser->YYData->{sel}= [ 'a', 'b', 'c' ];


my $text;

sub Lexer {
	my $parser = shift;
	while ($text !~ /^\s*$/)
	{
		if ($text=~s/^(\w+)//i)
		{
			return ('IDENTIFIER',$1);
		}
		elsif ($text=~s/^([\(\)\&\|\-\,])//)
		{
			my $token = $1;
			my $q_token = quotemeta($token);
			# -- cancels
			if (($token eq '-') and ($text=~s/^\s*-//))
			{
				next;
			}
			# ||... and &&... are similar with | or &
			elsif ($token =~ /^[\&\|\,]$/) 
			{
				while ($text =~ /^\s*$q_token/)
				{
					$text = ~s/^\sw*$q_token//;
					print STDERR "Warning: multiple $token\n";
				}
			}
			return ($token,$token);
		}
		elsif ($text=~s/^\s+//)
		{
		}
		else
		{
			$text=~s/^(.)//;
			print STDERR "Warning: unknown char in conditional: $1\n";
		}
	}
}

sub sel
{
	if (!defined($_[0]))
	{
		$parser->YYData->{sel}= [];
	}
	else
	{
		$parser->YYData->{sel}= [ @_ ];
	}
}


sub parse
{
	my $text_orig = shift;
	return undef if (!defined($text_orig));
	$text = $text_orig;
	my $result = $parser->YYParse(yylex => \&Lexer,
	#yyerror => \&ErrorReport
	);

	if (!defined($result))
	{
		print STDERR "Error parsing $text_orig\n";
	}
	return $result;
}

1;
